@@ -3,7 +3,7 @@
 Osmium -- OpenStreetMap data manipulation command line tool
 https://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2018  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2019  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -124,7 +124,7 @@ bool CommandDiff::setup(const std::vector<std::string>& arguments) {
             m_output_action = "compact";
         } else {
             m_output_action = "osm";
-            m_output_file = osmium::io::File(m_output_filename, m_output_format);
+            m_output_file = osmium::io::File{m_output_filename, m_output_format};
             m_output_file.check();
 
             auto f = m_output_file.format();
