@@ -3,7 +3,7 @@
 Osmium -- OpenStreetMap data manipulation command line tool
 https://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2018  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2019  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -26,6 +26,7 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 #include "export/export_format_json.hpp"
 #include "export/export_format_pg.hpp"
+#include "export/export_format_spaten.hpp"
 #include "export/export_format_text.hpp"
 #include "export/export_handler.hpp"
 
@@ -74,7 +75,7 @@ static std::string get_attr_string(const rapidjson::Value& object, const char* k
     return "";
 }
 
-void CommandExport::parse_options(const rapidjson::Value& attributes) {
+void CommandExport::parse_attributes(const rapidjson::Value& attributes) {
     if (!attributes.IsObject()) {
         throw config_error{"'attributes' member must be an object."};
     }
@@ -89,6 +90,38 @@ void CommandExport::parse_options(const rapidjson::Value& attributes) {
     m_options.way_nodes = get_attr_string(attributes, "way_nodes");
 }
 
+void CommandExport::parse_format_options(const rapidjson::Value& options) {
+    if (!options.IsObject()) {
+        throw config_error{"'format_options' member must be an object."};
+    }
+    for (const auto& kv : options.GetObject()) {
+        const auto type = kv.value.GetType();
+        const char* key = kv.name.GetString();
+        switch (type) {
+            case rapidjson::kNullType:
+                m_options.format_options.set(key, false);
+                break;
+            case rapidjson::kTrueType:
+                m_options.format_options.set(key, true);
+                break;
+            case rapidjson::kFalseType:
+                m_options.format_options.set(key, false);
+                break;
+            case rapidjson::kObjectType:
+                throw config_error{"Option value for key '" + std::string(key) + "' can not be of type object."};
+            case rapidjson::kArrayType:
+                throw config_error{"Option value for key '" + std::string(key) + "' can not be an array."};
+                break;
+            case rapidjson::kStringType:
+                m_options.format_options.set(key, kv.value.GetString());
+                break;
+            case rapidjson::kNumberType:
+                m_options.format_options.set(key, std::to_string(kv.value.GetInt64()));
+                break;
+        }
+    }
+}
+
 static Ruleset parse_tags_ruleset(const rapidjson::Value& object, const char* key) {
     Ruleset ruleset;
 
@@ -178,7 +211,12 @@ void CommandExport::parse_config_file() {
 
     const auto json_attr = doc.FindMember("attributes");
     if (json_attr != doc.MemberEnd()) {
-        parse_options(json_attr->value);
+        parse_attributes(json_attr->value);
+    }
+
+    const auto json_opts = doc.FindMember("format_options");
+    if (json_opts != doc.MemberEnd()) {
+        parse_format_options(json_opts->value);
     }
 
     m_linear_ruleset = parse_tags_ruleset(doc, "linear_tags");
@@ -223,6 +261,7 @@ bool CommandExport::setup(const std::vector<std::string>& arguments) {
     opts_cmd.add_options()
     ("add-unique-id,u", po::value<std::string>(), "Add unique id to each feature ('counter' or 'type_id')")
     ("config,c", po::value<std::string>(), "Config file")
+    ("format-option,x", po::value<std::vector<std::string>>(), "Output format options")
     ("fsync", "Call fsync after writing file")
     ("geometry-types", po::value<std::string>(), "Geometry types that should be written (default: 'point,linestring,polygon')")
     ("index-type,i", po::value<std::string>()->default_value(default_index_type), "Index type to use")
@@ -270,6 +309,8 @@ bool CommandExport::setup(const std::vector<std::string>& arguments) {
         "user":      false,
         "way_nodes": false
     },
+    "format_options": {
+    },
     "linear_tags":  true,
     "area_tags":    true,
     "exclude_tags": [],
@@ -291,6 +332,39 @@ bool CommandExport::setup(const std::vector<std::string>& arguments) {
     setup_progress(vm);
     setup_input_file(vm);
 
+    if (vm.count("output")) {
+        m_output_filename = vm["output"].as<std::string>();
+
+        const auto pos = m_output_filename.rfind('.');
+        if (pos != std::string::npos) {
+            m_output_format = m_output_filename.substr(pos + 1);
+        }
+    } else {
+        m_output_filename = "-";
+    }
+
+    if (vm.count("output-format")) {
+        m_output_format = vm["output-format"].as<std::string>();
+    }
+
+    canonicalize_output_format();
+
+    if (m_output_format != "geojson" &&
+        m_output_format != "geojsonseq" &&
+        m_output_format != "pg" &&
+        m_output_format != "text" &&
+        m_output_format != "spaten") {
+        throw argument_error{"Set output format with --output-format or -f to 'geojson', 'geojsonseq', 'pg', 'spaten', or 'text'."};
+    }
+
+    // Set defaults for output format options depending on output format
+    if (m_output_format == "geojsonseq") {
+        m_options.format_options.set("print_record_separator", true);
+    }
+    if (m_output_format == "pg") {
+        m_options.format_options.set("tags_type", "json");
+    }
+
     if (vm.count("config")) {
         m_config_file_name = vm["config"].as<std::string>();
 
@@ -349,33 +423,19 @@ bool CommandExport::setup(const std::vector<std::string>& arguments) {
         m_options.keep_untagged = true;
     }
 
-    if (vm.count("output")) {
-        m_output_filename = vm["output"].as<std::string>();
-
-        const auto pos = m_output_filename.rfind('.');
-        if (pos != std::string::npos) {
-            m_output_format = m_output_filename.substr(pos + 1);
-        }
-    } else {
-        m_output_filename = "-";
-    }
-
-    if (vm.count("output-format")) {
-        m_output_format = vm["output-format"].as<std::string>();
-    }
-
-    canonicalize_output_format();
-
-    if (m_output_format != "geojson" && m_output_format != "geojsonseq" && m_output_format != "pg" && m_output_format != "text") {
-        throw argument_error{"Set output format with --output-format or -f to 'geojson', 'geojsonseq', 'pg', or 'text'."};
-    }
-
     if (vm.count("overwrite")) {
         m_output_overwrite = osmium::io::overwrite::allow;
     }
 
+    if (vm.count("format-option")) {
+        for (const auto& str : vm["format-option"].as<std::vector<std::string>>()) {
+            m_options.format_options.set(str);
+        }
+    }
+
     if (vm.count("omit-rs")) {
-        m_options.print_record_separator = false;
+        m_options.format_options.set("print_record_separator", false);
+        warning("The --omit-rs/-r option is deprecated. Please use '-x print_record_separator=false' instead.\n");
         if (m_output_format != "geojsonseq") {
             warning("The --omit-rs/-r option only works for GeoJSON Text Sequence (geojsonseq) format. Ignored.\n");
         }
@@ -400,6 +460,10 @@ bool CommandExport::setup(const std::vector<std::string>& arguments) {
         initialize_tags_filter(m_options.tags_filter, true, m_exclude_tags);
     }
 
+    if (m_input_file.filename().empty()) {
+        throw config_error{"Can not read from STDIN, because input file has to be read twice."};
+    }
+
     return true;
 }
 
@@ -445,12 +509,7 @@ void CommandExport::show_arguments() {
 
     m_vout << "  output options:\n";
     m_vout << "    file name: " << m_output_filename << '\n';
-
-    if (m_output_format == "geojsonseq") {
-        m_vout << "    file format: geojsonseq (with" << (m_options.print_record_separator ? " RS)\n" : "out RS)\n");
-    } else {
-        m_vout << "    file format: " << m_output_format << '\n';
-    }
+    m_vout << "    file format: " << m_output_format << '\n';
     m_vout << "    overwrite: " << yes_no(m_output_overwrite == osmium::io::overwrite::allow);
     m_vout << "    fsync: " << yes_no(m_fsync == osmium::io::fsync::yes);
     m_vout << "  attributes:\n";
@@ -463,6 +522,13 @@ void CommandExport::show_arguments() {
     m_vout << "    user:      " << (m_options.user.empty()      ? "(omitted)" : m_options.user)      << '\n';
     m_vout << "    way_nodes: " << (m_options.way_nodes.empty() ? "(omitted)" : m_options.way_nodes) << '\n';
 
+    if (m_options.format_options.size() > 0) {
+        m_vout << "  output format options:\n";
+        for (const auto& option : m_options.format_options) {
+            m_vout << "    " << option.first << " = " << option.second << '\n';
+        }
+    }
+
     m_vout << "  linear tags: ";
     print_ruleset(m_vout, m_linear_ruleset);
     m_vout << "  area tags:   ";
@@ -499,10 +565,19 @@ static std::unique_ptr<ExportFormat> create_handler(const std::string& output_fo
         return std::unique_ptr<ExportFormat>{new ExportFormatText{output_format, output_filename, overwrite, fsync, options}};
     }
 
+    if (output_format == "spaten") {
+        return std::unique_ptr<ExportFormat>{new ExportFormatSpaten{output_format, output_filename, overwrite, fsync, options}};
+    }
+
     throw argument_error{"Unknown output format"};
 }
 
 bool CommandExport::run() {
+    auto handler = create_handler(m_output_format, m_output_filename, m_output_overwrite, m_fsync, m_options);
+    if (m_vout.verbose()) {
+        handler->debug_output(m_vout, m_output_filename);
+    }
+
     osmium::area::Assembler::config_type assembler_config;
     osmium::area::MultipolygonManager<osmium::area::Assembler> mp_manager{assembler_config};
 
@@ -511,15 +586,9 @@ bool CommandExport::run() {
     m_vout << "First pass done.\n";
 
     m_vout << "Second pass (of two) through input file...\n";
-
     m_linear_ruleset.init_filter();
     m_area_ruleset.init_filter();
 
-    auto handler = create_handler(m_output_format, m_output_filename, m_output_overwrite, m_fsync, m_options);
-    if (m_vout.verbose()) {
-        handler->debug_output(m_vout, m_output_filename);
-    }
-
     ExportHandler export_handler{std::move(handler), m_linear_ruleset, m_area_ruleset, m_geometry_types, m_show_errors, m_stop_on_error};
     osmium::handler::CheckOrder check_order_handler;
 
