@@ -6,7 +6,7 @@
 Osmium -- OpenStreetMap data manipulation command line tool
 https://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2018  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2019  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -63,7 +63,8 @@ class CommandExport : public Command, public with_single_osm_input {
     bool m_stop_on_error = false;
 
     void canonicalize_output_format();
-    void parse_options(const rapidjson::Value& attributes);
+    void parse_attributes(const rapidjson::Value& attributes);
+    void parse_format_options(const rapidjson::Value& options);
     void parse_config_file();
 
 public:
