@@ -3,7 +3,7 @@
 Osmium -- OpenStreetMap data manipulation command line tool
 https://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2018  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2019  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -180,8 +180,6 @@ struct InfoHandler : public osmium::handler::Handler {
 
 class Output {
 
-protected:
-
     bool m_calculate_crc = false;
 
 public:
@@ -196,6 +194,10 @@ public:
     Output(Output&&) noexcept = delete;
     Output& operator=(Output&&) noexcept = delete;
 
+    bool calculate_crc() const noexcept {
+        return m_calculate_crc;
+    }
+
     void set_crc(bool with_crc) noexcept {
         m_calculate_crc = with_crc;
     }
@@ -269,7 +271,7 @@ public:
             std::cout << "unknown (because objects in file are unordered)\n";
         }
 
-        if (m_calculate_crc) {
+        if (calculate_crc()) {
             std::cout << "  CRC32: " << std::hex << info_handler.crc32().checksum() << std::dec << "\n";
         } else {
             std::cout << "  CRC32: not calculated (use --crc/-c to enable)\n";
@@ -414,7 +416,7 @@ public:
             m_writer.Bool(info_handler.multiple_versions);
         }
 
-        if (m_calculate_crc) {
+        if (calculate_crc()) {
             m_writer.String("crc32");
             std::stringstream ss;
             ss << std::hex << info_handler.crc32().checksum() << std::dec;
@@ -826,11 +828,7 @@ bool CommandFileinfo::setup(const std::vector<std::string>& arguments) {
         if (m_get_value.substr(0, 5) == "data." && !m_extended) {
             throw argument_error{"You need to set --extended/-e for any 'data.*' variables to be available."};
         }
-        if (m_get_value == "data.crc32") {
-            m_calculate_crc = true;
-        } else {
-            m_calculate_crc = false;
-        }
+        m_calculate_crc = (m_get_value == "data.crc32");
     }
 
     if (vm.count("get") && vm.count("json")) {
