@@ -6,7 +6,7 @@
 Osmium -- OpenStreetMap data manipulation command line tool
 https://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2018  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2019  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -40,12 +40,14 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 class CommandGetId : public Command, public with_single_osm_input, public with_osm_output {
 
+    osmium::nwr_array<osmium::index::IdSetDense<osmium::unsigned_object_id_type>> m_matching_ids;
     osmium::nwr_array<osmium::index::IdSetDense<osmium::unsigned_object_id_type>> m_ids;
 
     osmium::item_type m_default_item_type = osmium::item_type::node;
 
     bool m_add_referenced_objects = false;
     bool m_work_with_history = false;
+    bool m_remove_tags = false;
     bool m_verbose_ids = false;
 
     osmium::osm_entity_bits::type get_needed_types() const;
