@@ -2,7 +2,7 @@
 Osmium -- OpenStreetMap data manipulation command line tool
 https://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2018  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2019  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -132,7 +132,7 @@ void id_map::read(int fd, std::size_t file_size) {
     }
 }
 
-static osmium::object_id_type get_start_id(const std::string& s) noexcept {
+static osmium::object_id_type get_start_id(const std::string& s) {
     const auto id = osmium::string_to_object_id(s.c_str());
     if (id == 0) {
         return 1;
@@ -379,6 +379,7 @@ bool CommandRenumber::run() {
     osmium::io::Header header = reader.header();
     setup_header(header);
     header.set("xml_josm_upload", "false");
+    header.set("sorting", "Type_then_ID");
 
     osmium::io::Writer writer{m_output_file, header, m_output_overwrite, m_fsync};
 
