@@ -3,7 +3,7 @@
 Osmium -- OpenStreetMap data manipulation command line tool
 https://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2018  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2019  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -154,6 +154,9 @@ void CommandTagsFilter::show_arguments() {
 
     m_vout << "  other options:\n";
     m_vout << "    add referenced objects: " << yes_no(m_add_referenced_objects);
+    if (m_add_referenced_objects) {
+        m_vout << "    remove tags on non-matching objects: " << yes_no(m_remove_tags);
+    }
     m_vout << "  looking for tags...\n";
     m_vout << "    on nodes: "     << yes_no(!m_filters(osmium::item_type::node).empty());
     m_vout << "    on ways: "      << yes_no(!m_filters(osmium::item_type::way).empty() || !m_area_filters.empty());
@@ -198,8 +201,8 @@ bool CommandTagsFilter::matches_node(const osmium::Node& node) const noexcept {
 
 bool CommandTagsFilter::matches_way(const osmium::Way& way) const noexcept {
     return osmium::tags::match_any_of(way.tags(), m_filters(osmium::item_type::way)) ||
-           (way.is_closed() &&
-            way.nodes().size() >= 4 &&
+           (way.nodes().size() >= 4 &&
+            way.is_closed() &&
                osmium::tags::match_any_of(way.tags(), m_area_filters));
 }
 
@@ -303,11 +306,11 @@ void CommandTagsFilter::find_nodes_in_ways() {
     osmium::io::Reader reader{m_input_file, osmium::osm_entity_bits::way};
     while (osmium::memory::Buffer buffer = reader.read()) {
         for (const auto& way : buffer.select<osmium::Way>()) {
-            if (m_referenced_ids(osmium::item_type::way).get(way.positive_id())) {
-                add_nodes(way);
-            } else if (matches_way(way) != m_invert_match) {
+            if (matches_way(way) != m_invert_match) {
                 m_matching_ids(osmium::item_type::way).set(way.positive_id());
                 add_nodes(way);
+            } else if (m_referenced_ids(osmium::item_type::way).get(way.positive_id())) {
+                add_nodes(way);
             }
         }
     }
@@ -332,21 +335,25 @@ void CommandTagsFilter::find_referenced_objects() {
 }
 
 bool CommandTagsFilter::run() {
+    m_vout << "Opening input file to get header...\n";
+    osmium::io::Reader reader_only_for_header{m_input_file, osmium::osm_entity_bits::nothing};
+
+    m_vout << "Opening output file...\n";
+    osmium::io::Header header{reader_only_for_header.header()};
+    setup_header(header);
+    reader_only_for_header.close();
+
+    osmium::io::Writer writer{m_output_file, header, m_output_overwrite, m_fsync};
+
     if (m_add_referenced_objects) {
         find_referenced_objects();
     }
 
     m_vout << "Opening input file...\n";
-    ++m_count_passes;
     osmium::io::Reader reader{m_input_file, get_needed_types()};
 
-    m_vout << "Opening output file...\n";
-    osmium::io::Header header = reader.header();
-    setup_header(header);
-
-    osmium::io::Writer writer{m_output_file, header, m_output_overwrite, m_fsync};
-
     m_vout << "Copying matching objects to output file...\n";
+    ++m_count_passes;
     osmium::ProgressBar progress_bar{reader.file_size(), display_progress()};
     while (osmium::memory::Buffer buffer = reader.read()) {
         progress_bar.update(reader.offset());
