@@ -3,7 +3,7 @@
 Osmium -- OpenStreetMap data manipulation command line tool
 http://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2018  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2019  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -20,6 +20,8 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 */
 
+#include "../exception.hpp"
+#include "../util.hpp"
 #include "export_format_pg.hpp"
 
 #include <osmium/io/detail/read_write.hpp>
@@ -28,18 +30,18 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #ifndef RAPIDJSON_HAS_STDSTRING
 # define RAPIDJSON_HAS_STDSTRING 1
 #endif
-#include <rapidjson/writer.h>
 #include <rapidjson/stringbuffer.h>
+#include <rapidjson/writer.h>
 
 #include <limits>
 #include <string>
 
 enum {
-    initial_buffer_size = 1024u * 1024u
+    initial_buffer_size = 1024U * 1024U
 };
 
 enum {
-    flush_buffer_size = 800u * 1024u
+    flush_buffer_size = 800U * 1024U
 };
 
 ExportFormatPg::ExportFormatPg(const std::string& /*output_format*/,
@@ -51,6 +53,15 @@ ExportFormatPg::ExportFormatPg(const std::string& /*output_format*/,
     m_fd(osmium::io::detail::open_for_writing(output_filename, overwrite)),
     m_fsync(fsync) {
     m_buffer.reserve(initial_buffer_size);
+
+    const auto tt = options.format_options.get("tags_type");
+    if (tt == "hstore") {
+        m_tags_type = tags_output_format::hstore;
+    } else if (tt == "json" || tt == "jsonb") {
+        m_tags_type = tags_output_format::json;
+    } else {
+        throw config_error{"Unknown value for tags_format option: '" + tt + "'."};
+    }
 }
 
 void ExportFormatPg::flush_to_output() {
@@ -99,15 +110,7 @@ void ExportFormatPg::append_pg_escaped(const char* str, std::size_t size = std::
 
 void ExportFormatPg::add_attributes(const osmium::OSMObject& object) {
     if (!options().type.empty()) {
-        if (object.type() == osmium::item_type::area) {
-            if (static_cast<const osmium::Area&>(object).from_way()) {
-                m_buffer.append("way");
-            } else {
-                m_buffer.append("relation");
-            }
-        } else {
-            m_buffer.append(osmium::item_type_to_name(object.type()));
-        }
+        m_buffer.append(object_type_as_string(object));
         m_buffer += '\t';
     }
 
@@ -161,7 +164,7 @@ void ExportFormatPg::add_attributes(const osmium::OSMObject& object) {
     }
 }
 
-bool ExportFormatPg::add_tags(const osmium::OSMObject& object) {
+bool ExportFormatPg::add_tags_json(const osmium::OSMObject& object) {
     bool has_tags = false;
 
     rapidjson::StringBuffer stream;
@@ -182,6 +185,55 @@ bool ExportFormatPg::add_tags(const osmium::OSMObject& object) {
     return has_tags;
 }
 
+static void add_escape_hstore(std::string& out, const char* str) {
+    out += "\"";
+
+    while (*str) {
+        if (*str == '"') {
+            out += "\\\"";
+        } else if (*str == '\\') {
+            out += "\\\\";
+        } else {
+            out += *str;
+        }
+        ++str;
+    }
+
+    out += "\"";
+}
+
+bool ExportFormatPg::add_tags_hstore(const osmium::OSMObject& object) {
+    if (object.tags().empty()) {
+        return false;
+    }
+
+    bool has_tags = false;
+
+    std::string data;
+
+    for (const auto& tag : object.tags()) {
+        if (options().tags_filter(tag)) {
+            has_tags = true;
+            add_escape_hstore(data, tag.key());
+            data += "=>";
+            add_escape_hstore(data, tag.value());
+            data += ',';
+        }
+    }
+
+    if (has_tags) {
+        data.resize(data.size() - 1);
+        append_pg_escaped(data.c_str());
+    }
+
+    return has_tags;
+}
+
+bool ExportFormatPg::add_tags(const osmium::OSMObject& object) {
+    return m_tags_type == tags_output_format::json ? add_tags_json(object)
+                                                   : add_tags_hstore(object);
+}
+
 void ExportFormatPg::finish_feature(const osmium::OSMObject& object) {
     m_buffer += '\t';
     add_attributes(object);
@@ -230,19 +282,23 @@ void ExportFormatPg::close() {
 
 void ExportFormatPg::debug_output(osmium::VerboseOutput& out, const std::string& filename) {
     out << '\n';
+
     out << "Create table with something like this:\n";
+    if (m_tags_type == tags_output_format::hstore) {
+        out << "CREATE EXTENSION IF NOT EXISTS hstore;\n";
+    }
     out << "CREATE TABLE osmdata (\n";
 
     if (options().unique_id == unique_id_type::counter) {
         out << "    id        BIGINT PRIMARY KEY,\n";
     } else if (options().unique_id == unique_id_type::type_id) {
-        out << "    id        VARCHAR PRIMARY KEY,\n";
+        out << "    id        TEXT PRIMARY KEY,\n";
     }
 
-    out << "    geom      GEOMETRY,\n";
+    out << "    geom      GEOMETRY, -- or GEOGRAPHY\n";
 
     if (!options().type.empty()) {
-        out << "    osm_type  VARCHAR,\n";
+        out << "    osm_type  TEXT,\n";
     }
 
     if (!options().id.empty()) {
@@ -262,7 +318,7 @@ void ExportFormatPg::debug_output(osmium::VerboseOutput& out, const std::string&
     }
 
     if (!options().user.empty()) {
-        out << "    user      VARCHAR,\n";
+        out << "    user      TEXT,\n";
     }
 
     if (!options().timestamp.empty()) {
@@ -273,7 +329,14 @@ void ExportFormatPg::debug_output(osmium::VerboseOutput& out, const std::string&
         out << "    way_nodes BIGINT[],\n";
     }
 
-    out << "    tags      JSON -- or JSONB\n";
+    switch (m_tags_type) {
+        case tags_output_format::json:
+            out << "    tags      JSONB -- or JSON, or TEXT\n";
+            break;
+        case tags_output_format::hstore:
+            out << "    tags      hstore\n";
+            break;
+    }
     out << ");\n";
     out << "Then load data with something like this:\n";
     out << "\\copy osmdata FROM '" << filename << "'\n";
