@@ -3,7 +3,7 @@
 Osmium -- OpenStreetMap data manipulation command line tool
 https://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2018  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2019  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -20,6 +20,7 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 */
 
+#include "../util.hpp"
 #include "export_format_text.hpp"
 
 #include <osmium/io/detail/read_write.hpp>
@@ -61,15 +62,7 @@ void ExportFormatText::add_attributes(const osmium::OSMObject& object) {
     if (!options().type.empty()) {
         m_buffer.append(options().type);
         m_buffer.append(1, '=');
-        if (object.type() == osmium::item_type::area) {
-            if (static_cast<const osmium::Area&>(object).from_way()) {
-                m_buffer.append("way");
-            } else {
-                m_buffer.append("relation");
-            }
-        } else {
-            m_buffer.append(osmium::item_type_to_name(object.type()));
-        }
+        m_buffer.append(object_type_as_string(object));
         m_buffer.append(1, ',');
     }
 
