@@ -3,7 +3,7 @@
 Osmium -- OpenStreetMap data manipulation command line tool
 https://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2018  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2019  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -27,18 +27,6 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #include <sstream>
 #include <string>
 
-namespace osmium {
-
-    namespace io {
-        class Header;
-    } // namespace io
-
-    namespace memory {
-        class Item;
-    } // namespace memory
-
-} // namespace osmium
-
 void Extract::open_file(const osmium::io::Header& header, osmium::io::overwrite output_overwrite, osmium::io::fsync sync) {
     m_writer.reset(new osmium::io::Writer{m_output_file, header, output_overwrite, sync});
 }
