@@ -6,7 +6,7 @@
 Osmium -- OpenStreetMap data manipulation command line tool
 https://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2018  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2019  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -25,12 +25,8 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 #include "extract.hpp"
 
-namespace osmium {
-
-    class Area;
-    class Segment;
-
-} // namespace osmium
+#include <osmium/osm/area.hpp>
+#include <osmium/osm/segment.hpp>
 
 #include <vector>
 
