@@ -3,7 +3,7 @@
 Osmium -- OpenStreetMap data manipulation command line tool
 https://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2018  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2019  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -100,7 +100,15 @@ void parse_rings(const rapidjson::Value& value, osmium::builder::AreaBuilder& bu
         const auto outer_ring = parse_ring(array[0]);
         osmium::builder::OuterRingBuilder ring_builder{builder};
         for (const auto& c : outer_ring) {
-            ring_builder.add_node_ref(0, osmium::Location{c.x, c.y});
+            osmium::Location loc{c.x, c.y};
+            if (loc.valid()) {
+                ring_builder.add_node_ref(0, loc);
+            } else {
+                throw config_error{"Invalid location in boundary (multi)polygon: (" +
+                                   std::to_string(c.x) +
+                                   ", " +
+                                   std::to_string(c.y) + ")."};
+            }
         }
     }
 
@@ -108,7 +116,15 @@ void parse_rings(const rapidjson::Value& value, osmium::builder::AreaBuilder& bu
         const auto inner_ring = parse_ring(array[i]);
         osmium::builder::InnerRingBuilder ring_builder{builder};
         for (const auto& c : inner_ring) {
-            ring_builder.add_node_ref(0, osmium::Location{c.x, c.y});
+            osmium::Location loc{c.x, c.y};
+            if (loc.valid()) {
+                ring_builder.add_node_ref(0, loc);
+            } else {
+                throw config_error{"Invalid location in boundary (multi)polygon: (" +
+                                   std::to_string(c.x) +
+                                   ", " +
+                                   std::to_string(c.y) + ")."};
+            }
         }
     }
 }
@@ -142,7 +158,7 @@ std::size_t parse_multipolygon_array(const rapidjson::Value& value, osmium::memo
     return buffer.commit();
 }
 
-OSMIUM_NORETURN void GeoJSONFileParser::error(const std::string& message) {
+[[noreturn]] void GeoJSONFileParser::error(const std::string& message) {
     throw geojson_error{std::string{"In file '"} + m_file_name + "':\n" + message};
 }
 
