@@ -6,7 +6,7 @@
 Osmium -- OpenStreetMap data manipulation command line tool
 https://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2018  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2019  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -23,15 +23,7 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 */
 
-#include <osmium/util/compatibility.hpp>
-
-namespace osmium {
-
-    namespace memory {
-        class Buffer;
-    } // namespace memory
-
-} // namespace osmium
+#include <osmium/memory/buffer.hpp>
 
 #include <rapidjson/document.h>
 
@@ -51,7 +43,7 @@ class GeoJSONFileParser {
     std::string m_file_name;
     std::ifstream m_file;
 
-    OSMIUM_NORETURN void error(const std::string& message);
+    [[noreturn]] void error(const std::string& message);
 
     std::size_t parse_top(const rapidjson::Value& top);
 
