@@ -3,7 +3,7 @@
 Osmium -- OpenStreetMap data manipulation command line tool
 https://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2018  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2019  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -20,6 +20,7 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 */
 
+#include "../exception.hpp"
 #include "osm_file_parser.hpp"
 
 #include <osmium/area/assembler.hpp>
@@ -55,7 +56,7 @@ std::size_t OSMFileParser::operator()() {
     }
 
     bool has_ring = false;
-    {
+    try {
         index_type index;
         location_handler_type location_handler{index};
         osmium::builder::AreaBuilder builder{m_buffer};
@@ -73,6 +74,10 @@ std::size_t OSMFileParser::operator()() {
             }
         }));
         reader.close();
+    } catch (const osmium::invalid_location&) {
+        throw config_error{"Invalid location in boundary (multi)polygon in '" + m_file_name + "'."};
+    } catch (const osmium::not_found&) {
+        throw config_error{"Missing node in boundary (multi)polygon in '" + m_file_name + "'."};
     }
 
     if (has_ring) {
