@@ -6,7 +6,7 @@
 Osmium -- OpenStreetMap data manipulation command line tool
 https://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2018  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2019  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -23,13 +23,7 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 */
 
-namespace osmium {
-
-    namespace memory {
-        class Buffer;
-    } // namespace memory
-
-} // namespace osmium
+#include <osmium/memory/buffer.hpp>
 
 #include <string>
 
