@@ -3,7 +3,7 @@
 Osmium -- OpenStreetMap data manipulation command line tool
 https://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2018  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2019  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -212,3 +212,16 @@ void with_osm_output::setup_header(osmium::io::Header& header) const {
     }
 }
 
+void with_osm_output::setup_header(osmium::io::Header& header, const osmium::io::Header& input_header) const {
+    header.set("generator", m_generator);
+    for (const auto& h : m_output_headers) {
+        if (!h.empty() && h.back() == '!') {
+            std::string hc{h};
+            hc.resize(h.size() - 1);
+            header.set(hc, input_header.get(hc));
+        } else {
+            header.set(h);
+        }
+    }
+}
+
