@@ -4,11 +4,11 @@
 #include "command_cat.hpp"
 
 TEST_CASE("cat") {
-
-    CommandCat cmd;
+    CommandFactory factory;
+    CommandCat cmd{factory};
 
     SECTION("no parameters") {
-        REQUIRE_THROWS_AS(cmd.setup({}), argument_error);
+        REQUIRE_THROWS_AS(cmd.setup({}), const argument_error&);
     }
 
     SECTION("stdin osm to stdout pbf") {
@@ -18,22 +18,22 @@ TEST_CASE("cat") {
         REQUIRE(ifs.size() == 1);
         REQUIRE(ifs.front().format() == osmium::io::file_format::xml);
         REQUIRE(ifs.front().compression() == osmium::io::file_compression::none);
-        REQUIRE(ifs.front().filename() == "");
+        REQUIRE(ifs.front().filename().empty());
 
         auto of = cmd.output_file();
         REQUIRE(of.format() == osmium::io::file_format::pbf);
-        REQUIRE(of.filename() == "");
+        REQUIRE(of.filename().empty());
 
         REQUIRE(cmd.output_overwrite() == osmium::io::overwrite::no);
         REQUIRE(cmd.osm_entity_bits() == osmium::osm_entity_bits::all);
     }
 
     SECTION("unknown object-type 1") {
-        REQUIRE_THROWS_AS(cmd.setup({"in.osm", "-o", "out.osm", "-t", "foo"}), argument_error);
+        REQUIRE_THROWS_AS(cmd.setup({"in.osm", "-o", "out.osm", "-t", "foo"}), const argument_error&);
     }
 
     SECTION("unknown object-type 2") {
-        REQUIRE_THROWS_AS(cmd.setup({"in.osm", "-o", "out.osm", "--object-type", "foo"}), argument_error);
+        REQUIRE_THROWS_AS(cmd.setup({"in.osm", "-o", "out.osm", "--object-type", "foo"}), const argument_error&);
     }
 
     SECTION("reading to files") {
@@ -57,23 +57,23 @@ TEST_CASE("cat") {
     }
 
     SECTION("unknown input format suffix") {
-        REQUIRE_THROWS_AS(cmd.setup({"in.foo"}), argument_error);
+        REQUIRE_THROWS_AS(cmd.setup({"in.foo"}), const argument_error&);
     }
 
     SECTION("unknown input format option") {
-        REQUIRE_THROWS_AS(cmd.setup({"in.osm", "-F", "foo"}), argument_error);
+        REQUIRE_THROWS_AS(cmd.setup({"in.osm", "-F", "foo"}), const argument_error&);
     }
 
     SECTION("unknown output format suffix") {
-        REQUIRE_THROWS_AS(cmd.setup({"in.osm", "-o", "foo.bar"}), std::runtime_error);
+        REQUIRE_THROWS_AS(cmd.setup({"in.osm", "-o", "foo.bar"}), const std::runtime_error&);
     }
 
     SECTION("unknown output suffix and unknown format option") {
-        REQUIRE_THROWS_AS(cmd.setup({"in.osm", "-o", "foo.foo", "-f", "bar"}), std::runtime_error);
+        REQUIRE_THROWS_AS(cmd.setup({"in.osm", "-o", "foo.foo", "-f", "bar"}), const std::runtime_error&);
     }
 
     SECTION("unknown output format option") {
-        REQUIRE_THROWS_AS(cmd.setup({"in.osm", "-o", "foo.pbf", "-f", "bar"}), std::runtime_error);
+        REQUIRE_THROWS_AS(cmd.setup({"in.osm", "-o", "foo.pbf", "-f", "bar"}), const std::runtime_error&);
     }
 
 }
